# ncgmp09_CreateDatabase.py
#   Python script to create an empty NCGMP09-style
#   ArcGIS 9.3 geodatabase for geologic map data
#
#   Ralph Haugerud, USGS
#
#   Takes 4 arguments: <outputDir>  <geoDataBaseName> <coordSystem> <number of cross sections>
#     to use local directory, use # for outputDir
#     # accepted for <coordSystem>
#     if geoDatabaseName ends in .gdb, a file geodatabase will be created
#     if geoDatabaseName ends in .mdb, a personal geodatabase will be created
#	coordSystem is a filename for an ESRI coordinate system definition (look 
#	in directory arcgis/Coordinate Systems). Use # to define coordinate system
#     later
#
#   Requires that ncgmp09_definition.py be present in the local directory 
#     or in the appropriate Python library directory.
#
#   On my laptop, this takes many minutes to run and creates a 
#      ~0.5GB file/directory
#
# To use this:
#   	1) run script, e.g.
#		C:\myworkspace> ncgmp09_CreateDatabase.py # newgeodatabase.gdb # 1
#	2) Open newgeodatabase in ArcCatalog, doubleclick on feature data set GeologicMap,
#	   and assign a spatial reference system
#	3) If you use the CorrelationOfMapUnits feature data set, note that you will have to 
#	   manually create the annotation feature class CMUText and add field ParagraphStyle.
#	   (I haven't yet found a way to script the creation of an annotation feature class.)
#	4) If there will be non-standard point feature classes (photos, mineral occurrences,
#	   etc.), copy/paste/rename feature class GenericPoint or GenericSample, as appropriate,
#	   and add necessary fields to new feature class
#	5) Delete any unneeded feature classes and feature data sets
#	6) Load data. Edit as needed
#
#  NOTE: CAN ALSO BE RUN AS TOOLBOX SCRIPT FROM ARCCATALOG

import arcgisscripting, sys, os
from ncgmp09_definition import tableDict

versionString = 'ncgmp09_create.py, version of 15 October 2010'

default = '#'
cartoReps = True # False if cartographic representations will not be used

transDict =     { 'String': 'TEXT',
			'Single': 'FLOAT',
			'Double': 'DOUBLE',
			'NoNulls':'NON_NULLABLE',
			'NullsOK':'NULLABLE',
			'Date'  : 'DATE'  }

usage = """Usage:
   systemprompt> ncgmp09_create.py <directory> <geodatabaseName> <coordSystem> <#XSections>
   <directory> is existing directory in which new geodatabaseName is to 
      be created, use # for current directory
   <geodatabaseName> is name of gdb to be created, with extension
      .gdb causes a file geodatabase to be created
      .mdb causes a personal geodatabase to be created
   <coordSystem> is a fully-specified ArcGIS coordinate system
   <#XSections> is an integer (0, 1, 2, ...) specifying the intended number of cross-sections

  Then, in ArcCatalog:
  * If you use the CorrelationOfMapUnits feature data set, note that you will 
    have to manually create the annotation feature class CMUText and add field 
    ParagraphStyle. (I haven't yet found a way to script the creation of an 
    annotation feature class.)
  * If there will be non-standard point feature classes (photos, mineral 
    occurrences, etc.), copy/paste/rename feature class GenericPoint or 
    GenericSample, as appropriate, rename the _ID field, and add necessary
    fields to the new feature class.
  * Delete any unneeded feature classes and feature data sets.
  * Load data, if data already are in GIS form. 
  Edit data as needed.

"""

def addMsgAndPrint(gp,msg, severity=0): 
    # prints msg to screen and adds msg to the geoprocessor (in case this is run as a tool) 
    # print msg 
    try: 
        for string in msg.split('\n'): 
            # Add appropriate geoprocessing message 
            if severity == 0: 
                gp.AddMessage(string) 
            elif severity == 1: 
                gp.AddWarning(string) 
            elif severity == 2: 
                gp.AddError(string) 
    except: 
        pass 

def createFeatureClass(gp,thisDB,featureDataSet,featureClass,shapeType,fieldDefs):
    addMsgAndPrint(gp,'    Creating feature class '+featureClass+'...')
    try:
        gp.CreateFeatureClass_Management(thisDB+'/'+featureDataSet,featureClass,shapeType)
    except:
        addMsgAndPrint(gp,'Failed to create feature class '+featureClass+' in dataset '+featureDataSet)
        addMsgAndPrint(gp,gp.GetMessages(2))
    thisFC = thisDB+'/'+featureDataSet+'/'+featureClass
    for fDef in fieldDefs:
        try:
            if fDef[1] == 'String':
                gp.AddField_Management(thisFC,fDef[0],transDict[fDef[1]],'#','#',fDef[3],'#',transDict[fDef[2]])
            else:
                gp.AddField_Management(thisFC,fDef[0],transDict[fDef[1]],'#','#','#','#',transDict[fDef[2]])
        except:
            addMsgAndPrint(gp,'Failed to add field '+fDef[0]+' to feature class '+featureClass)
            addMsgAndPrint(gp,gp.GetMessages(2))

        
def main(gp,thisDB,coordSystem,nCrossSections):
	# create feature dataset GeologicMap
	addMsgAndPrint(gp,'  Creating feature dataset GeologicMap...')
	try:
		gp.CreateFeatureDataset_management(thisDB,'GeologicMap')
	except:
		addMsgAndPrint(gp,gp.GetMessages(2))
	if coordSystem <> '#':
		try:
		    gp.DefineProjection_management(thisDB+'/GeologicMap',coordSystem)
		except:
		    addMsgAndPrint(gp,gp.GetMessages(2)) 

	# create feature classes in GeologicMap
	# poly feature classes
	featureClasses = ['MapUnitPolys','DataSourcePolys','OtherPolys']
	for featureClass in featureClasses:
		fieldDefs = tableDict[featureClass]
		createFeatureClass(gp,thisDB,'GeologicMap',featureClass,'POLYGON',fieldDefs)

	# line feature classes
	featureClasses = ['ContactsAndFaults','GeologicLines','CartographicLines','IsoValueLines']
	for featureClass in featureClasses:
		fieldDefs = tableDict[featureClass]
		createFeatureClass(gp,thisDB,'GeologicMap',featureClass,'POLYLINE',fieldDefs)

	# point feature classes
	featureClasses = ['OrientationPoints','GeochronPoints','FossilPoints','MapUnitPoints','Stations',
		      'GenericSamples','GenericPoints']
	for featureClass in featureClasses:
		if featureClass == 'MapUnitPoints': 
			fieldDefs = tableDict['MapUnitPolys']
		else:	
			fieldDefs = tableDict[featureClass]
		createFeatureClass(gp,thisDB,'GeologicMap',featureClass,'POINT',fieldDefs)

	# create feature dataset CorrelationOfMapUnits
	addMsgAndPrint(gp,'  Creating feature dataset CorrelationOfMapUnits...')
	gp.CreateFeatureDataset_management(thisDB,'CorrelationOfMapUnits')
	fieldDefs = tableDict['CMUMapUnitPolys']
	createFeatureClass(gp,thisDB,'CorrelationOfMapUnits','CMUMapUnitPolys','POLYGON',fieldDefs)
	fieldDefs = tableDict['CMULines']
	createFeatureClass(gp,thisDB,'CorrelationOfMapUnits','CMULines','POLYLINE',fieldDefs)
	fieldDefs = tableDict['CMUPoints']
	createFeatureClass(gp,thisDB,'CorrelationOfMapUnits','CMUPoints','POINT',fieldDefs)
	
	# create CrossSections
	if nCrossSections > 26:
		nCrossSections = 26
	if nCrossSections < 0:
		nCrossSections = 0
	# note space in position 0
	alphabet = ' ABCDEFGHIJKLMNOPQRSTUVWXYZ'
	
	for n in range(1,nCrossSections+1):
		xsLetter = alphabet[n]
		xsName = 'CrossSection'+xsLetter
		xsN = 'CS'+xsLetter
		#create feature dataset CrossSectionA
		addMsgAndPrint(gp,'  Creating feature data set CrossSection'+xsLetter+'...')
		gp.CreateFeatureDataset_management(thisDB,xsName)
		fieldDefs = tableDict['MapUnitPolys']
		fieldDefs[0][0] = xsN+'MapUnitPolys_ID'
		createFeatureClass(gp,thisDB,xsName,xsN+'MapUnitPolys','POLYGON',fieldDefs)
		fieldDefs = tableDict['ContactsAndFaults']
		fieldDefs[0][0] = xsN+'ContactsAndFaults_ID'
		createFeatureClass(gp,thisDB,xsName,xsN+'ContactsAndFaults','POLYLINE',fieldDefs)
		fieldDefs = tableDict['OrientationPoints']
		fieldDefs[0][0] = xsN+'OrientationPoints_ID'
		createFeatureClass(gp,thisDB,xsName,xsN+'OrientationPoints','POINT',fieldDefs)

	# create tables
	tables = ['DescriptionOfMapUnits','DataSources','Glossary',
      	    'RepurposedSymbols','StandardLithology','ExtendedAttributes','GeologicEvents']
	for table in tables:
		addMsgAndPrint(gp,'  Creating table '+table+'...')
		try:
			gp.CreateTable(thisDB,table)
		except:
			addMsgAndPrint(gp,gp.GetMessages(2))
		fieldDefs = tableDict[table]
		for fDef in fieldDefs:
			try:
			  if fDef[1] == 'String':
				gp.AddField_Management(thisDB+'/'+table,fDef[0],transDict[fDef[1]],'#','#',fDef[3],'#',transDict[fDef[2]])
			  else:
				gp.AddField_Management(thisDB+'/'+table,fDef[0],transDict[fDef[1]],'#','#','#','#',transDict[fDef[2]])
			except:
				addMsgAndPrint(gp,'Failed to add field '+fDef[0]+' to table '+table)
				addMsgAndPrint(gp,gp.GetMessages(2))

        # if cartoReps = True, add cartographic representations to all feature classses
        if cartoReps:
            gp.workspace = thisDB
            datasets = gp.listdatasets()
            for dataset in datasets:
                gp.workspace = thisDB+'/'+dataset
                fcs = gp.ListFeatureClasses()
                for fc in fcs:
                    addMsgAndPrint(gp,'  Adding cartographic representations to '+fc)
                    try:
                        gp.addRepresentation_cartography(fc,fc+'_rep','RuleID','Override',default,default,'NO_ASSIGN')
                    except:
                        addMsgAndPrint(gp,gp.GetMessages(2))

                       

def createDatabase(gp,outputDir,thisDB):
    addMsgAndPrint(gp,'  Creating geodatabase '+thisDB+'...')		
    try:
        if thisDB[-4:] == '.mdb':
            gp.createPersonalGDB_management(outputDir,thisDB)
        if thisDB[-4:] == '.gdb':
            gp.createFileGDB_management(outputDir,thisDB)
            try:
                versionFile = open(outputDir+'/'+thisDB+'/00readme.txt','write')
                versionFile.write('Geodatabase created by '+versionString+'\n')
                versionFile.close()
            except:
                addMsgAndPrint(gp,'Failed to write '+outputDir+'/'+thisDB+'/00readme.txt')
        return True
    except:
        addMsgAndPrint(gp,'Failed to create geodatabase '+outputDir+'/'+thisDB)
        addMsgAndPrint(gp,gp.GetMessages(2))
        return False


#########################################
if len(sys.argv) == 5:
    try:
        gp = arcgisscripting.create(9.3)
        outputDir = sys.argv[1]
        thisDB = sys.argv[2]
        # test for extension; if not given, default to file geodatabase
        if not thisDB[-4:].lower() in ('.gdb','.mdb'):
            thisDB = thisDB+'.gdb'
        coordSystem = sys.argv[3]
        nCrossSections = int(sys.argv[4])

        if outputDir == '#':
            outputDir = os.getcwd()

        # create personal gdb in output directory
        if createDatabase(gp,outputDir,thisDB):
            thisDB = outputDir+'/'+thisDB
            gp.refreshcatalog(thisDB)
            main(gp,thisDB,coordSystem,nCrossSections)
    except:
	addMsgAndPrint(gp,'Failed. Perhaps could not create geoprocessing object?')
else:
    addMsgAndPrint(gp,usage)
